package com.ctgu.pro_eld_mqtt_compare.mqtt;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.channel.DirectChannel;
import org.springframework.integration.mqtt.core.DefaultMqttPahoClientFactory;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.integration.mqtt.outbound.MqttPahoMessageHandler;
import org.springframework.messaging.MessageChannel;

import java.util.UUID;

@Configuration
public class MqttOutboundConfiguration {

    private final MqttOutboundProperties mqttProperties;

    @Autowired
    public MqttOutboundConfiguration(MqttOutboundProperties mqttProperties) {
        this.mqttProperties = mqttProperties;
    }

    @Bean
    public MqttPahoClientFactory mqttOutboundClientFactory() {
        DefaultMqttPahoClientFactory factory = new DefaultMqttPahoClientFactory();
        factory.getConnectionOptions().setServerURIs(new String[]{mqttProperties.getUrl()});
        factory.getConnectionOptions().setUserName(mqttProperties.getUsername());
        factory.getConnectionOptions().setPassword(mqttProperties.getPassword().toCharArray());
        factory.getConnectionOptions().setKeepAliveInterval(30);
        factory.getConnectionOptions().setAutomaticReconnect(true);
        factory.getConnectionOptions().setCleanSession(false);
        return factory;
    }

    @Bean
    public MessageChannel mqttOutboundChannel() {
        return new DirectChannel();
    }

    @Bean
    @ServiceActivator(inputChannel = "mqttOutboundChannel")
    public MqttPahoMessageHandler mqttOutboundHandler() {
        MqttPahoMessageHandler handler = new MqttPahoMessageHandler(
                mqttProperties.getUrl(),
                mqttProperties.getClientId() + UUID.randomUUID(),
                mqttOutboundClientFactory()
        );
        handler.setDefaultQos(mqttProperties.getQos());
        handler.setDefaultTopic(mqttProperties.getTopic());
        handler.setAsync(true);
        return handler;
    }
}
